# -*- coding: utf-8 -*-
"""
Created on Wed Jun  2 08:52:09 2021

@author: MkZee
"""

import mcvqoe.mouth2ear.m2e as m2e
import mcvqoe.mouth2ear

from .shared import TestCfgFrame, iterations
from .shared import advanced as shared_advanced
from .shared import AdvancedConfigGUI, MultiChoice, SignalOverride, test

import csv
import numpy as np
  
#------------------------------- Controls -------------------------------------

from .shared import audio_files, audio_path
from .shared import BgNoise, SaveAudio, dev_dly
from .shared import trials, ptt_wait, ptt_gap, outdir


class M2EAdvancedConfigGUI(AdvancedConfigGUI):
    text = 'M2E Latency - Advanced'
    
    def get_controls(self):
        return (
            BgNoise,
            )


class advanced(shared_advanced):
    toplevel = M2EAdvancedConfigGUI


# -------------------------- The mouth-to-ear frame ---------------------------

class M2eFrame(TestCfgFrame):
    
    text = 'Mouth-to-Ear Latency Test'
        
    def get_controls(self):
        return (
            audio_files,
            audio_path,
            outdir,
            trials,
            iterations,
            ptt_wait,
            ptt_gap,
            SaveAudio,
            dev_dly,
            test,
            advanced
            )
    

# --------------------- using the m2e measure class ---------------------------

class M2E_fromGui(SignalOverride, m2e.measure):

    def get_mean_and_std(self, name=None):
        
        if(not name):
            name = self.data_filename
        
        with open(name, 'rt') as csv_f:
            # create dict reader
            reader = csv.DictReader(csv_f)
            # empty list for M2E data
            m2e_dat = []
            #
            for row in reader:
                m2e_dat.append(float(row['m2e_latency']))

        # convert to numpy array
        m2e_dat = np.array(m2e_dat)
        
        # Overall mean delay
        ovrl_dly = np.mean(m2e_dat)
        
        # Get standard deviation
        std_delay = np.std(m2e_dat, dtype=np.float64)
        std_delay = std_delay*(1e6)
        
        return ovrl_dly, std_delay
        

class M2E_Eval_from_GUI(SignalOverride, mcvqoe.mouth2ear.evaluate):
    pass


#-----------------------------Dev dly characterization------------------------

class DevDlyCharFrame(M2eFrame):
    
    text = 'Device Delay Characterization'
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        
        # disable all parameters except for outdir
        for k, v in self.controls.items():
            if k != 'outdir':
                if hasattr(v, 'm_ctrl'):
                    try:
                        v.m_ctrl['state'] = 'disabled'
                    except: pass
                    
                if hasattr(v, 'r_ctrl'):
                    try:
                        v.r_ctrl.configure(state='disabled')
                    except: pass


# contains the default values for the measurement
class DevChar_Defaults(m2e.measure):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        
        self.test     = '1loc'
        self.trials   = 400
        self.ptt_wait = 0.001
        self.ptt_gap  = 0.31
        

class DevChar_fromGui(M2E_fromGui): pass



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 12 12:26:08 2021

@author: jkp4
"""
from dash import dcc
from dash import html
from dash.dependencies import Input, Output, State

import json
import numpy as np
import os
import pandas as pd

from mcvqoe.hub.eval_app import app

import mcvqoe.hub.eval_shared as eval_shared



#-----------------------[Begin layout]---------------------------
# TODO: Say something about common thinning fctor if data can't be thined

measurement = 'm2e'
layout = eval_shared.layout_template(measurement)

    
def format_m2e_results(m2e_eval, digits=6):
    """
    Format results from mouth2ear.evaluate object to be in nice HTML.

    Parameters
    ----------
    m2e_eval : mcvqoe.mouth2ear.evaluate
        DESCRIPTION.

    Returns
    -------
    children : html.Div
        DESCRIPTION.

    """
    if np.isnan(m2e_eval.common_thinning):
        thin_msg = f'No common thinning factor identified, statistics displayed for unthinned data'
    else:
        thin_msg = f'{m2e_eval.common_thinning}'
    pretty_mean = eval_shared.pretty_numbers(m2e_eval.mean, digits)
    pretty_ci = eval_shared.pretty_numbers(m2e_eval.ci, digits)
    children = html.Div([
        html.H6('Mean mouth-to-ear latency'),
        html.Div(f'{pretty_mean} seconds'),
        html.H6('95% Confidence Interval'),
        html.Div(f'{pretty_ci} seconds'),
        html.H6('Thinning factor'),
        html.Div(thin_msg),
        ],
        style=eval_shared.style_results,
        # className='six columns',
        )
    return children

# --------------[Callback functions (order matters here!)]--------------------
@app.callback(
    Output(f'{measurement}-measurement-results', 'children'),
    Output(f'{measurement}-measurement-formatting', 'children'),
    Output(f'{measurement}-scatter', 'figure'),
    Output(f'{measurement}-hist', 'figure'),
    Output(f'{measurement}-talker-select', 'options'),
    Output(f'{measurement}-session-select', 'options'),
    Input(f'{measurement}-json-data', 'data'),
    Input(f'{measurement}-thin-select', 'value'),
    Input(f'{measurement}-talker-select', 'value'),
    Input(f'{measurement}-session-select', 'value'),
    Input(f'{measurement}-x-axis', 'value'),
    Input(f'{measurement}-measurement-digits', 'value'),
    )
def update_plots(jsonified_data, thin, talker_select, session_select, x, meas_digits):
    """
    Update all plots

    Parameters
    ----------
    jsonified_data : TYPE
        DESCRIPTION.
    thin : TYPE
        DESCRIPTION.
    talker_select : TYPE
        DESCRIPTION.
    session_select : TYPE
        DESCRIPTION.
    x : TYPE
        DESCRIPTION.

    Returns
    -------
    return_vals : TYPE
        DESCRIPTION.

    """
    
    if jsonified_data is not None:
        try:
            json_data = json.loads(jsonified_data)
            if 'error' in json_data:
                error_out = '. '.join(json_data['error'])
                raise RuntimeError(error_out)
            else:
                m2e_eval = eval_shared.load_json_data(jsonified_data, f'{measurement}')

            thinned = thin == 'True'
            if x == 'index':
                x = None
            if talker_select == []:
                talker_select = None
            if session_select == []:
                session_select = None
            
            fig_scatter = m2e_eval.plot(
                x=x,
                thinned=thinned,
                talkers=talker_select,
                test_name=session_select,
                color_palette=eval_shared.plotly_color_palette,
                )
            fig_histogram = m2e_eval.histogram(
                thinned=thinned,
                talkers=talker_select,
                test_name=session_select,
                color_palette=eval_shared.plotly_color_palette,
                )
            
            talkers = np.unique(m2e_eval.data['Filename'])
            talker_options = [{'label': i, 'value': i} for i in talkers]
            
            sessions = m2e_eval.test_names
            session_options = [{'label': i, 'value': i} for i in sessions]
            
            res = format_m2e_results(m2e_eval, meas_digits)
            res_formatting = eval_shared.measurement_digits('grid', meas_digits,
                                                            measurement=measurement)
            return_vals = (
                res,
                res_formatting,
                fig_scatter,
                fig_histogram,
                talker_options,
                session_options
                )
        except Exception as e:
            print(e)
            return_vals = eval_shared.failed_process(measurement, msg=e.args)
        
    else:
        return_vals = eval_shared.failed_process(measurement, )
        
    return return_vals
        


        


# -*- coding: utf-8 -*-
"""
Created on Thu Jul  8 16:22:33 2021

@author: MkZee
"""

import mcvqoe.intelligibility as igtiby

from .shared import TestCfgFrame, SignalOverride
from .shared import advanced as shared_advanced
from .shared import AdvancedConfigGUI, test

#--------------------------Controls-------------------------------------------

from .shared import BgNoise, trials, outdir, ptt_wait
from .shared import ptt_gap, RadioCheck, SaveAudio, iterations

class intell_trials(trials):
    """
Estimated errors and test time for different number of tests.

Simulations will run faster than estimated times.

N	RMS Deviation	Maximum Absolute Deviation	Estimated Test Time
16	    0.017	           0.053	            1.75 minutes
32	    0.009	           0.033                    3.5 minutes
64	    0.005	           0.014	            7 minutes
128	    0.003	           0.008	            14 minutes
256	    0.002	           0.004                    28 minutes
512         0.001                  0.002	            56 minutes
1200        0.000                  0.000	            131 minutes
"""
    __doc__ = trials.__doc__ + __doc__


# -------------------- The advanced configuration frame -----------------------

class IntellAdvancedConfigGUI(AdvancedConfigGUI):
    text = 'Intelligibility - Advanced'

    def get_controls(self):
        return (
            BgNoise,
            )


class advanced(shared_advanced):
    toplevel = IntellAdvancedConfigGUI

# ---------------------- The main configuration frame -------------------------

class IgtibyFrame(TestCfgFrame):
    text = 'Intelligibility Test'
    def get_controls(self):
        return (
            outdir,
            intell_trials,
            iterations,
            ptt_wait,
            ptt_gap,
            SaveAudio,
            RadioCheck,
            test,
            advanced,
            )

# ---------------------- Extending the measure class --------------------------

class Igtiby_from_Gui(SignalOverride, igtiby.measure):
    pass

class Intell_eval_from_GUI(SignalOverride, igtiby.evaluate):
    def param_check(self):
        # future proof
        if hasattr(super(), 'param_check'):
            super().param_check()


#!/usr/bin/env python
import setuptools

with open("README.md", "r",encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="mcvqoe",
    author="PSCR",
    author_email="PSCR@PSCR.gov",
    description="Graphical interface for mcvqoe measurements",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/usnistgov/mcvqoe",
    packages=setuptools.find_namespace_packages(include=['mcvqoe.*']),
    include_package_data=True,
    use_scm_version={'write_to' : 'mcvqoe/hub/version.py'},
    setup_requires=['setuptools_scm'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Public Domain",
        "Operating System :: OS Independent",
    ],
    license='NIST software License',
    install_requires=[
        'abcmrt16',
        'mcvqoe-base',
        'mcvqoe-psud',
        'mcvqoe-intelligibility',
        'mcvqoe-accesstime',
        'mcvqoe-mouth2ear',
        'mcvqoe-tvo',
        'numpy',
        'pandas',
        'dash',
        'flask',
        'plotly',
        'requests',
        'pillow',
        'scipy',
        'werkzeug<2.1'
    ],
    entry_points={
        'gui_scripts':[
            'mcvqoe=mcvqoe.hub.mcv_qoe_gui:main',
        ],
        'console_scripts':[
            'mcvqoe-eval=mcvqoe.hub.eval_index:main',
            ]
    },
    python_requires='>=3.8',
)




# -*- coding: utf-8 -*-
"""
Created on Thu Nov 18 13:04:07 2021

@author: jkp4
"""
import os
import mcvqoe.tvo as tvo
import tkinter as tk
import tkinter.filedialog as fdl

from .shared import SubCfgFrame, TestCfgFrame, AdvancedConfigGUI, SignalOverride
from .shared import LabeledNumber, LabeledCheckbox, LabeledSlider
from .shared import format_audio_files, _get_master
from .shared import advanced as shared_advanced

#----------------------------controls-----------------------------------------

from .shared import trials, audio_files, audio_path, outdir
from .shared import ptt_wait, ptt_gap, time_expand
from .shared import SaveAudio, MultiChoice, iterations


class ptt_rep(LabeledNumber):
    """Number of times to repeat at a given volume level."""
    
    text = 'Repeats per volume level:'


class dev_volume(LabeledNumber):
    """
    Volume level of audio output from the audio interface. This is typically
    set using controls specific to individual audio interfaces, and cannot be
    set programatically.
    """
    max_=0
    min_=-100
    text = "Audio Interface Output Volume (dB)"
    

class smax(LabeledNumber):
    """Maximum number of sample volumes to use."""
    
    text = 'Max sample values'


class tol(LabeledNumber):
    """
    The minimum volume difference between any two evaluated points. Smaller 
    values results in more iterations, larger values results in a coarser 
    optimal volume estimate.
    """
    min_ = 0
    increment = 0.01
    text = 'Volume tolerance (dB):'

# ----------------Volume Levels---------------------------


class Volumes(SubCfgFrame):
    
    text = 'Volume Levels'
    toggleable_controls = [
        '_num_volumes',
        ]
    def __init__(self, master, *args, **kwargs):
        
        super().__init__(master, *args, **kwargs)
        
        btv = self.btnvars['_enable_Fixed_Volumes']
        trace_id = btv.trace_add('write',lambda *a,**k:self.update())
        
        # mark the trace for deletion, otherwise causes errors
        _get_master(self).traces_.append((btv, trace_id))
        
        self.update()
    def get_controls(self):
        return (
            _enable_Fixed_Volumes,
            _min_volume,
            _max_volume,
            _num_volumes,
            )
    
    def update(self):
        
        state = ('disabled', '!disabled')[self.btnvars['_enable_Fixed_Volumes'].get()]
        
        # disable other controls
        for ctrlname in self.toggleable_controls:
            ctrl = self.controls[ctrlname]
            
        # for ctrlname, ctrl in self.controls.items():
        #     if ctrlname == '_enable_Fixed_Volumes':
        #         continue
            
            ctrl.m_ctrl.configure(state=state)


class _enable_Fixed_Volumes(LabeledCheckbox):
    """
    Use fixed volumes defined by minimum, maximum, and number of volumes below.
    If enabled, an array with the input amount of uniformly spaced volumes 
    between the minimum and maximum quantity will be used for evaluation.
    
    If disabled, evaluation volumes are determined algorithmically during a test.
    Initial volumes for consideration will be between the minimum and maximum 
    volumes set here.
    """

    text = 'Use fixed volume levels'


class _min_volume(LabeledNumber):
    """
    TODO: write this
    """
    max_ = 0
    text = "Minimum volume to evaluate"


class _max_volume(LabeledNumber):
    """
    TODO: Write this
    """
    max_ = 0
    text = "Maximum volume to evaluate"


class _num_volumes(LabeledNumber):
    """
    Number of volumes to evaluate if use fixed volumes selected
    """
    min_ = 2
    max_ = 30
    text = "Number of volumes"

# ---------------------- The main config frame --------------------------------


class TVOFrame(TestCfgFrame):
    
    text = 'Transmit Volume Optimization Procedure'
    
    def get_controls(self):
        return (
            audio_files,
            audio_path,
            outdir,
            iterations,
            SaveAudio,
            ptt_wait,
            ptt_gap,
            advanced,
            )
    

class TVOAdvanced(AdvancedConfigGUI):
    
    text = 'TVO - Advanced'
    
    def get_controls(self):
        return (
            dev_volume,
            ptt_rep,
            smax,
            tol,
            Volumes,
            )


class advanced(shared_advanced):
    toplevel = TVOAdvanced
    
#-------------------------Running the test------------------------------------


class TVO_fromGui(SignalOverride, tvo.measure):
    
    def param_check(self):
        # future-proofing this param-check override
        if hasattr(super(), 'param_check'):
            super().param_check()

class TVO_eval_from_GUI(SignalOverride, tvo.evaluate):
    pass